/* 
 * Copyright (C) 2005  Christian Cier-Zniewski
 * Author: Christian Cier-Zniewski <c.cier@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <pthread.h>
#include <unistd.h>
#include <sys/socket.h>
#include <errno.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <signal.h>
#include <fcntl.h>
#include <stdio.h>
#include <memory.h>
#include <netdb.h>
#include <stdlib.h>

#include "global.h"

sigset_t     signal_set;
pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
int interrupted = 0;

/*
 * Wait for the SIGINT signal. When it has occurred 5 times, set
 * the "interrupted" flag (the main thread's wait predicate) and
 * signal a condition variable. The main thread will exit.
 */
void *signal_waiter (void *arg)
{
    int sig_number;
    int status;

    while (1) {
        sigwait (&signal_set, &sig_number);
        if (sig_number == SIGINT) {
            printf ("Got SIGINT \n");
            status = pthread_mutex_lock (&mutex);
            if (status != 0)
                    fprintf(stderr, "Lock mutex");
            interrupted = 1;
            status = pthread_cond_signal (&cond);
            if (status != 0)
                    fprintf(stderr, "Signal condition");
            status = pthread_mutex_unlock (&mutex);
            if (status != 0)
                    fprintf(stderr, "Unlock mutex");
                break;
            }
     }
    return NULL;
}


int
main (int argc,char **argv)
{

   pthread_t    signal_thread_id;
   int status;
   
   sigemptyset ( &signal_set );
   sigaddset ( &signal_set, SIGINT );
   
   status = pthread_sigmask (SIG_BLOCK, &signal_set, NULL);
   if (status != 0)
   {
       fprintf(stderr, "ERROR: Set signal mask");
       exit(0);
   }
   /*
    * Create the sigwait thread.
   */
   status = pthread_create (&signal_thread_id, NULL, signal_waiter, NULL);
   if (status != 0)
   {
       fprintf(stderr,"ERROR: Create sigwaiter");
       exit(0);
   }

   /* Connect to server (RFB) */
   if (!(rfbsock=tcp_connect_to_server(SERVERNAME, RFBSERVERPORT)))
   {
      printf("Cannot establish connection with server (OSD Receiver). Exiting.\n");
      close(rfbsock);
      exit(0);
   }
   
   /* Connect to server (TS Receiver)*/
   if (!(tsrecvsock=tcp_connect_to_server(SERVERNAME, STREAMPORT)))
   {
      printf("Cannot establish connection with server (TS Receiver). Exiting.\n");
      close(tsrecvsock);
      exit(0);
   }

   StartTSreceiver();
   StartOSDreceiver();

   fprintf(stderr,"Entering main loop...\n");	
   /*
    * Wait for the sigwait thread to receive SIGINT and signal
    * the condition variable.
    */
   status = pthread_mutex_lock (&mutex);
   if (status != 0)
       fprintf(stderr, "Lock mutex");
   while (!interrupted) {
       status = pthread_cond_wait (&cond, &mutex);
       if (status != 0)
           fprintf(stderr, "Wait for interrupt");
   }
   status = pthread_mutex_unlock (&mutex);
   if (status != 0)
       fprintf(stderr, "Unlock mutex");
   printf ("Main loop terminating with SIGINT\n");
   
   StopOSDreceiver();
   StopTSreceiver();
   
   close (tsrecvsock);
   close (rfbsock);
      
   return 0;
}
   