/* 
 * Copyright (C) 2005  Christian Cier-Zniewski
 * Author: Christian Cier-Zniewski <c.cier@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <pthread.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <signal.h>
#include <fcntl.h>
#include <stdio.h>
#include <memory.h>
#include <netdb.h>
#include <stdlib.h>

#include "ost/dmx.h"
#include "ost/video.h"
#include "ost/audio.h"

#include "global.h"
#include "ringbuffy.h"

#define BUFFERSIZE 2048000

static volatile int TSreceiver_active;
pthread_t TSreceiver_thread;

//struct ringbuffy ringbuf;

int dvrfd, vfd, afd, videofd, audiofd;

void set_pid(int fd, int pid, int type)
{
        struct dmxPesFilterParams pesFilterParams;

        fprintf(stderr, "set PID 0x%04x (%d)\n", pid, type);
        if (ioctl(fd, DMX_STOP) < 0)
                perror("DMX STOP:");

      if (ioctl(fd, DMX_SET_BUFFER_SIZE, 64*1024) < 0)
              perror("DMX SET BUFFER:");

        pesFilterParams.pid = pid;
        pesFilterParams.input = DMX_IN_DVR;
        pesFilterParams.output = DMX_OUT_DECODER;
        pesFilterParams.pesType = type;
        pesFilterParams.flags = DMX_IMMEDIATE_START;
        if (ioctl(fd, DMX_SET_PES_FILTER, &pesFilterParams) < 0)
                perror("DMX SET FILTER");
}

int pvr_init(int vpid, int apid)
{
        char *viddev = "/dev/dvb/card0/video0";
        char *auddev = "/dev/dvb/card0/audio0";
        char *dmxdev = "/dev/dvb/card0/demux0";
        char *dvrdev = "/dev/pvr";

        if ((dvrfd = open(dvrdev, O_WRONLY)) == -1) {   //|O_NONBLOCK
                fprintf(stderr, "Failed to open '%s': %d %m\n", dvrdev, errno);
                return 0;
        }

        if ((vfd = open(dmxdev, O_WRONLY)) == -1) {
                fprintf(stderr, "Failed to open video demux '%s': %d %m\n", dmxdev, errno);
                return 0;
        }
        if ((afd = open(dmxdev, O_WRONLY)) == -1) {
                fprintf(stderr, "Failed to open audio demux '%s': %d %m\n", dmxdev, errno);
                return 0;
        }

        if ((videofd = open(viddev, O_WRONLY)) == -1) {
                fprintf(stderr, "Failed to open video device'%s': %d %m\n", viddev, errno);
                return 0;
        }

        if ((audiofd = open(auddev, O_WRONLY)) == -1) {
                fprintf(stderr, "Failed to open audio device'%s': %d %m\n", auddev, errno);
                return 0;
        }

        if (ioctl(videofd, VIDEO_PLAY) < 0)
                perror("VIDEO_PLAY:");

        if (ioctl(audiofd, AUDIO_SET_AV_SYNC, 1) < 0)
                perror("AUDIO_PLAY:");

        if (ioctl(audiofd, AUDIO_PLAY) < 0)
                perror("AUDIO_PLAY:");

        /* playback timing is controlled via A/V PTS, so we cannot start
         * writing to the DVR device before the PIDs are set...
         */

        set_pid(afd, apid, DMX_PES_AUDIO);
        set_pid(vfd, vpid, DMX_PES_VIDEO);
    
        return 1;
}



// TS-over-TCP receiver thread
void 
*TSreceiver(void *threadid)
{
   int numbytes;
   //int written;
   //int bytesread;
   //int total;
   //int prebuffering=1;
   char buffer[100*188];
   	
	TSreceiver_active = 1;
	
	while (TSreceiver_active) 
	{
		if ((numbytes = tcp_getdata(tsrecvsock, buffer, 100*188))>0)
		{
			write(dvrfd, buffer, numbytes);
        	}    	

/*
		//fprintf(stderr,"Buffering... %d\n",ring_rest(&ringbuf));
		if ((numbytes = tcp_getdata(tsrecvsock, buffer, 100*188))>0)
		{
			if ( ring_write(&ringbuf, buffer, numbytes) !=numbytes)
				fprintf(stderr, "Ringbuffer full!\n");
#if 0 
			fprintf(stderr,"Got %d bytes.\n",numbytes);
        		for (numbytes=0; numbytes<4; numbytes++)
	       				fprintf(stderr,"%02X ",TSbuffer[numbytes]);
	   		fprintf(stderr,"\n");
#endif	   		
        	}    	
		if ( (ring_rest(&ringbuf)<100*188) && (prebuffering==1)) 
		{
			prebuffering = 0;
			//fprintf(stderr, "Buffering stop...%d\n", ring_rest(&ringbuf));
		}
			
		if (prebuffering==0) {
			bytesread = ring_read(&ringbuf, buffer, 100*188);
			if (bytesread == 0) 
			{
				//prebuffering = 1;
				//fprintf(stderr, "Buffering start...%d\n", ring_rest(&ringbuf));
			}
			total = 0;
			while ( total < bytesread ) 
			{
				written = write(dvrfd, buffer+total, bytesread-total);
				total += written;
			}
		}
*/
     	}

	fprintf(stderr,"TSreceiver thread terminated (id:%x).\r\n", (int) threadid);
	fflush(stderr);
	pthread_exit(NULL);	
}

void
StartTSreceiver(void)
{
   
   int rc;
   
   pvr_init(99,100); // Video PID 99, Audio PID 100
   //ring_init(&ringbuf, BUFFERSIZE);
   //fprintf(stderr,"Rest: %d\n",ring_rest(&ringbuf));
   
   fprintf(stderr,"Creating TSreceiver thread (id: 0x01).\n");
   rc = pthread_create(&TSreceiver_thread, NULL, TSreceiver, (void *) 1);
   if (rc){
      fprintf(stderr,"ERROR; return code from pthread_create() is %d\n", rc);
      exit(-1);
   }
}

void
StopTSreceiver(void)
{
   int rc, status;
   
   TSreceiver_active = 0;
   rc = pthread_join(TSreceiver_thread, (void **)&status);
   if (rc)
   {
      printf("ERROR; return code from pthread_join() is %d\n", rc);
      exit(-1);
   }
   fprintf(stderr,"TSreceiver thread joined.\r\n" );
   
   //ring_destroy(&ringbuf);
   close(dvrfd);
   close(afd);
   close(vfd);
   close(audiofd);
   close(videofd);
}

